@@ -18,6 +18,7 @@ Key manager implementation for Barbican
 """
 import calendar
 import time
+import urllib
 
 from cryptography.hazmat import backends
 from cryptography.hazmat.primitives import serialization
@@ -32,11 +33,6 @@ from oslo_utils import excutils
 from castellan.common import exception
 from castellan.common.objects import key as key_base_class
 from castellan.common.objects import opaque_data as op_data
-from castellan.common.objects import passphrase
-from castellan.common.objects import private_key as pri_key
-from castellan.common.objects import public_key as pub_key
-from castellan.common.objects import symmetric_key as sym_key
-from castellan.common.objects import x_509
 from castellan.i18n import _
 from castellan.key_manager import key_manager
 
@@ -44,10 +40,9 @@ from castellan.key_manager import key_manager
 from barbicanclient import client as barbican_client_import
 from barbicanclient import exceptions as barbican_exceptions
 from oslo_utils import timeutils
-from six.moves import urllib
 
 
-barbican_opts = [
+_barbican_opts = [
     cfg.StrOpt('barbican_endpoint',
                help='Use this endpoint to connect to Barbican, for example: '
                     '"http://localhost:9311/"'),
@@ -70,9 +65,15 @@ barbican_opts = [
                 default=True,
                 help='Specifies if insecure TLS (https) requests. If False, '
                      'the server\'s certificate will not be validated'),
+    cfg.StrOpt('barbican_endpoint_type',
+               default='public',
+               choices=['public', 'internal', 'admin'],
+               help='Specifies the type of endpoint.  Allowed values are: '
+                    'public, private, and admin'),
+
 ]
 
-BARBICAN_OPT_GROUP = 'barbican'
+_BARBICAN_OPT_GROUP = 'barbican'
 
 LOG = logging.getLogger(__name__)
 
@@ -80,20 +81,12 @@ LOG = logging.getLogger(__name__)
 class BarbicanKeyManager(key_manager.KeyManager):
     """Key Manager Interface that wraps the Barbican client API."""
 
-    _secret_type_dict = {
-        op_data.OpaqueData: 'opaque',
-        passphrase.Passphrase: 'passphrase',
-        pri_key.PrivateKey: 'private',
-        pub_key.PublicKey: 'public',
-        sym_key.SymmetricKey: 'symmetric',
-        x_509.X509: 'certificate'}
-
     def __init__(self, configuration):
         self._barbican_client = None
         self._base_url = None
         self.conf = configuration
-        self.conf.register_opts(barbican_opts, group=BARBICAN_OPT_GROUP)
-        loading.register_session_conf_options(self.conf, BARBICAN_OPT_GROUP)
+        self.conf.register_opts(_barbican_opts, group=_BARBICAN_OPT_GROUP)
+        loading.register_session_conf_options(self.conf, _BARBICAN_OPT_GROUP)
 
     def _get_barbican_client(self, context):
         """Creates a client to connect to the Barbican service.
@@ -138,7 +131,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         return self._barbican_client
 
     def _get_keystone_auth(self, context):
-        if context.__class__.__name__ is 'KeystonePassword':
+        if context.__class__.__name__ == 'KeystonePassword':
             return identity.Password(
                 auth_url=context.auth_url,
                 username=context.username,
@@ -154,7 +147,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
                 project_domain_id=context.project_domain_id,
                 project_domain_name=context.project_domain_name,
                 reauthenticate=context.reauthenticate)
-        elif context.__class__.__name__ is 'KeystoneToken':
+        elif context.__class__.__name__ == 'KeystoneToken':
             return identity.Token(
                 auth_url=context.auth_url,
                 token=context.token,
@@ -168,14 +161,17 @@ class BarbicanKeyManager(key_manager.KeyManager):
                 reauthenticate=context.reauthenticate)
         # this will be kept for oslo.context compatibility until
         # projects begin to use utils.credential_factory
-        elif context.__class__.__name__ is 'RequestContext':
-            return identity.Token(
-                auth_url=self.conf.barbican.auth_endpoint,
-                token=context.auth_token,
-                project_id=context.project_id,
-                project_name=context.project_name,
-                project_domain_id=context.project_domain_id,
-                project_domain_name=context.project_domain_name)
+        elif context.__class__.__name__ == 'RequestContext':
+            if getattr(context, 'get_auth_plugin', None):
+                return context.get_auth_plugin()
+            else:
+                return identity.Token(
+                    auth_url=self.conf.barbican.auth_endpoint,
+                    token=context.auth_token,
+                    project_id=context.project_id,
+                    project_name=context.project_name,
+                    project_domain_id=context.project_domain_id,
+                    project_domain_name=context.project_domain_name)
         else:
             msg = _("context must be of type KeystonePassword, "
                     "KeystoneToken, or RequestContext.")
@@ -183,18 +179,28 @@ class BarbicanKeyManager(key_manager.KeyManager):
             raise exception.Forbidden(reason=msg)
 
     def _get_barbican_endpoint(self, auth, sess):
-        if self.conf.barbican.barbican_endpoint:
-            return self.conf.barbican.barbican_endpoint
+        barbican = self.conf.barbican
+        if barbican.barbican_endpoint:
+            return barbican.barbican_endpoint
+        elif getattr(auth, 'service_catalog', None):
+            endpoint_data = auth.service_catalog.endpoint_data_for(
+                service_type='key-manager')
+            return endpoint_data.url
         else:
             service_parameters = {'service_type': 'key-manager',
                                   'service_name': 'barbican',
-                                  'interface': 'public'}
+                                  'interface': barbican.barbican_endpoint_type}
             return auth.get_endpoint(sess, **service_parameters)
 
     def _create_base_url(self, auth, sess, endpoint):
+        api_version = None
         if self.conf.barbican.barbican_api_version:
             api_version = self.conf.barbican.barbican_api_version
-        else:
+        elif getattr(auth, 'service_catalog', None):
+            endpoint_data = auth.service_catalog.endpoint_data_for(
+                service_type='key-manager')
+            api_version = endpoint_data.api_version
+        elif getattr(auth, 'get_discovery', None):
             discovery = auth.get_discovery(sess, url=endpoint)
             raw_data = discovery.raw_version_data()
             if len(raw_data) == 0:
@@ -615,7 +621,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
         except (barbican_exceptions.HTTPAuthError,
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
-            LOG.error(_("Error listing objects: %s"), e)
+            LOG.error("Error listing objects: %s", e)
             raise exception.KeyManagerError(reason=e)
 
         for secret in secrets:
@@ -625,7 +631,10 @@ class BarbicanKeyManager(key_manager.KeyManager):
             except (barbican_exceptions.HTTPAuthError,
                     barbican_exceptions.HTTPClientError,
                     barbican_exceptions.HTTPServerError) as e:
-                LOG.warning(_("Error occurred while retrieving object "
-                              "metadata, not adding it to the list: %s"), e)
+                LOG.warning("Error occurred while retrieving object "
+                            "metadata, not adding it to the list: %s", e)
 
         return objects
+
+    def list_options_for_discovery(self):
+        return [(_BARBICAN_OPT_GROUP, _barbican_opts)]
